@@ -7,7 +7,7 @@
 #include <sys/ioctl.h>
 #include "zaptel.h"
 #include "tonezone.h"
-#include "wcfxs.h"
+#include "wctdm.h"
 
 static int tones[] = {
 	ZT_TONE_DIALTONE,
@@ -28,6 +28,7 @@ int main(int argc, char *argv[])
 		       "       stats - reports voltages\n"
 		       "       regdump - dumps ProSLIC registers\n"
 		       "       tones - plays a series of tones\n"
+		       "       polarity - tests polarity reversal\n"
 		       "       ring - rings phone\n");
 		exit(1);
 	}
@@ -46,6 +47,24 @@ int main(int argc, char *argv[])
 			fprintf(stderr, "Phone is ringing...\n");
 			sleep(2);
 		}
+	} else if (!strcasecmp(argv[2], "polarity")) {
+		fprintf(stderr, "Twiddling polarity...\n");
+		x = 0;
+		res = ioctl(fd, ZT_SETPOLARITY, &x);
+		if (res) {
+			fprintf(stderr, "Unable to polarity...\n");
+		} else {
+			fprintf(stderr, "Polarity is forward...\n");
+			sleep(2);
+			x = 1;
+			ioctl(fd, ZT_SETPOLARITY, &x);
+			fprintf(stderr, "Polarity is reversed...\n");
+			sleep(5);
+			x = 0;
+			ioctl(fd, ZT_SETPOLARITY, &x);
+			fprintf(stderr, "Polarity is forward...\n");
+			sleep(2);
+		}
 	} else if (!strcasecmp(argv[2], "tones")) {
 		int x = 0;
 		for (;;) {
@@ -56,8 +75,8 @@ int main(int argc, char *argv[])
 			x=(x+1) % (sizeof(tones) / sizeof(tones[0]));
 		}
 	} else if (!strcasecmp(argv[2], "stats")) {
-		struct wcfxs_stats stats;
-		res = ioctl(fd, WCFXS_GET_STATS, &stats);
+		struct wctdm_stats stats;
+		res = ioctl(fd, WCTDM_GET_STATS, &stats);
 		if (res) {
 			fprintf(stderr, "Unable to get stats on channel %s\n", argv[1]);
 		} else {
@@ -66,10 +85,10 @@ int main(int argc, char *argv[])
 			printf("VBAT: %7.4f Volts\n", (float)stats.batvolt / 1000.0);
 		}
 	} else if (!strcasecmp(argv[2], "regdump")) {
-		struct wcfxs_regs regs;
+		struct wctdm_regs regs;
 		int numregs = NUM_REGS;
 		memset(&regs, 0, sizeof(regs));
-		res = ioctl(fd, WCFXS_GET_REGS, &regs);
+		res = ioctl(fd, WCTDM_GET_REGS, &regs);
 		if (res) {
 			fprintf(stderr, "Unable to get registers on channel %s\n", argv[1]);
 		} else {
@@ -97,7 +116,7 @@ int main(int argc, char *argv[])
 		}
 	} else if (!strcasecmp(argv[2], "setdirect") ||
 				!strcasecmp(argv[2], "setindirect")) {
-		struct wcfxs_regop regop;
+		struct wctdm_regop regop;
 		int val;
 		int reg;
 		if ((argc < 5) || (sscanf(argv[3], "%i", &reg) != 1) ||
@@ -112,7 +131,7 @@ int main(int argc, char *argv[])
 			} else {
 				regop.indirect = 0;
 			}
-			res = ioctl(fd, WCFXS_SET_REG, &regop);
+			res = ioctl(fd, WCTDM_SET_REG, &regop);
 			if (res) 
 				fprintf(stderr, "Unable to get registers on channel %s\n", argv[1]);
 			else
