@@ -2,7 +2,6 @@
 #include <fcntl.h>
 #include <string.h>
 #include <errno.h>
-#include <linux/zaptel.h>
 #include <stdio.h>
 #include <linux/types.h>
 #include <linux/ppp_defs.h> 
@@ -11,6 +10,12 @@
 #include <stdlib.h>
 #include "bittest.h"
 
+#ifdef STANDALONE_ZAPATA
+#include "zaptel.h"
+#else
+#include <linux/zaptel.h>
+#endif
+
 #define BLOCK_SIZE 2039
 
 void print_packet(unsigned char *buf, int len)
