@@ -1807,15 +1807,18 @@ static void __exit pciradio_cleanup(void)
 	pci_unregister_driver(&pciradio_driver);
 }
 
+#ifdef LINUX26
+module_param(debug, int, 0600);
+#else
 MODULE_PARM(debug, "i");
-MODULE_DESCRIPTION("Zapate Telephony PCI Radio Card Zaptel Driver");
+#endif
+
+MODULE_DESCRIPTION("Zapata Telephony PCI Radio Card Zaptel Driver");
 MODULE_AUTHOR("Jim Dixon <jim@lambdatel.com>");
+
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
 #endif
 
 module_init(pciradio_init);
 module_exit(pciradio_cleanup);
-
-
-
