@@ -7,7 +7,12 @@
 #include <sys/select.h>
 #include <sys/time.h>
 #include <errno.h>
+
+#ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
+#else
+#include <linux/zaptel.h>
+#endif
 
 int main(int argc, char *argv[])
 {
@@ -43,7 +48,7 @@ int main(int argc, char *argv[])
 			exit(1);
 		}
 		gettimeofday(&now, NULL);
-		printf("Timer Expired (%d ms)!\n", (now.tv_sec - orig.tv_sec) * 1000 + (now.tv_usec - orig.tv_usec) / 1000);
+		printf("Timer Expired (%ld ms)!\n", (now.tv_sec - orig.tv_sec) * 1000 + (now.tv_usec - orig.tv_usec) / 1000);
 	}
 	exit(0);
 }
