@@ -130,7 +130,11 @@ static int prefmaster = 0;
 
 static int spansstarted = 0;
 
+#ifdef DEFINE_RWLOCK
+static DEFINE_RWLOCK(torisa);
+#else
 static rwlock_t torisa = RW_LOCK_UNLOCKED;
+#endif
 
 static u_char readdata[2][64][ZT_MAX_CHUNKSIZE];
 static u_char writedata[2][64][ZT_MAX_CHUNKSIZE];
