@@ -31,6 +31,7 @@
 #include <linux/usb.h>
 #include <linux/errno.h>
 #include <linux/pci.h>
+#include <asm/io.h>
 #ifdef STANDALONE_ZAPATA
 #include "zaptel.h"
 #else
@@ -673,7 +674,7 @@ static int wcfxo_hardware_init(struct wcfxo *wc)
 	outb(0x01, wc->ioaddr + WC_CNTL);
 
 	/* Make sure serial port and DMA are out of reset */
-	outb(inb(wc->ioaddr + WC_CNTL) & 0xf9, WC_CNTL);
+	outb(inb(wc->ioaddr + WC_CNTL) & 0xf9, wc->ioaddr + WC_CNTL);
 
 	/* Configure serial port for MSB->LSB operation */
 	if (wc->flags & FLAG_DOUBLE_CLOCK)
@@ -1006,7 +1007,7 @@ static int __init wcfxo_init(void)
 			printk("%d: %s\n", x, fxo_modes[x].name);
 		return -ENODEV;
 	}
-	res = pci_module_init(&wcfxo_driver);
+	res = zap_pci_module(&wcfxo_driver);
 	if (res)
 		return -ENODEV;
 	return 0;
