@@ -1083,7 +1083,7 @@ static void wctdm_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 							wc->mod[x].fxs.idletxhookstate = 0x5;	/* Switch to active */
 						else
 							wc->mod[x].fxs.idletxhookstate = 0x1;
-						if ((wc->mod[x].fxs.lasttxhook == 0x2) || (wc->mod[x].fxs.lasttxhook = 0x6)) {
+						if ((wc->mod[x].fxs.lasttxhook == 0x2) || (wc->mod[x].fxs.lasttxhook == 0x6)) {
 							/* Apply the change if appropriate */
 							if (reversepolarity) 
 								wc->mod[x].fxs.lasttxhook = 0x5;
@@ -2332,6 +2332,7 @@ static struct pci_device_id wctdm_pci_tbl[] = {
 	{ 0xe159, 0x0001, 0xe159, PCI_ANY_ID, 0, 0, (unsigned long) &wctdm },
 	{ 0xe159, 0x0001, 0xb100, PCI_ANY_ID, 0, 0, (unsigned long) &wctdme },
 	{ 0xe159, 0x0001, 0xb1d9, PCI_ANY_ID, 0, 0, (unsigned long) &wctdmi },
+	{ 0xe159, 0x0001, 0xb118, PCI_ANY_ID, 0, 0, (unsigned long) &wctdmi },
 	{ 0xe159, 0x0001, 0xb119, PCI_ANY_ID, 0, 0, (unsigned long) &wctdmi },
 	{ 0xe159, 0x0001, 0xa9fd, PCI_ANY_ID, 0, 0, (unsigned long) &wctdmh },
 	{ 0xe159, 0x0001, 0xa8fd, PCI_ANY_ID, 0, 0, (unsigned long) &wctdmh },
