@@ -3,7 +3,7 @@
  *
  * Written by Mark Spencer <markster@digium.com>
  *
- * Copyright (C) 2005, Digium, Inc.
+ * Copyright (C) 2005, 2006, Digium, Inc.
  *
  * All rights reserved.
  *
@@ -387,7 +387,7 @@ struct wctdm_desc {
 	int flags;
 };
 
-static struct wctdm_desc wctdm = { "Wildcard TDM2400P Prototype", 0 };
+static struct wctdm_desc wctdm = { "Wildcard TDM2400P", 0 };
 static int acim2tiss[16] = { 0x0, 0x1, 0x4, 0x5, 0x7, 0x0, 0x0, 0x6, 0x0, 0x0, 0x0, 0x2, 0x0, 0x3 };
 
 static struct wctdm *ifaces[WC_MAX_IFACES];
@@ -1587,7 +1587,7 @@ static int wctdm_powerup_proslic(struct wctdm *wc, int card, int fast)
 	}
 
 	if (vbat < 0xc0) {
-		printk("ProSLIC on module %d failed to powerup within %d ms (%d mV only)\n\n -- DID YOU REMEMBER TO PLUG IN THE HD POWER CABLE TO THE TDM400P??\n",
+		printk("ProSLIC on module %d failed to powerup within %d ms (%d mV only)\n\n -- DID YOU REMEMBER TO PLUG IN THE HD POWER CABLE TO THE TDM2400P??\n",
 		       card, (int)(((jiffies - origjiffies) * 1000 / HZ)),
 			vbat * 375);
 		return -1;
