@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
  *
- * $Id: wcte11xp.c,v 1.1.2.8 2005/04/11 06:15:59 russell Exp $
+ * $Id: wcte11xp.c 837 2005-11-29 18:42:08Z kpfleming $
  */
 
 #include <linux/kernel.h>
@@ -143,7 +143,7 @@ struct t1 {
 	/* Our offset for finding channel 1 */
 	int offset;
 	char *variety;
-	int intcount;
+	unsigned int intcount;
 	int usecount;
 	int clocktimeout;
 	int sync;
@@ -178,6 +178,7 @@ struct t1 {
 #define CANARY 0xca1e
 
 static int debug = 0;   /* doesnt do anything */
+static int j1mode = 0;
 static int alarmdebounce = 0;
 static int loopback = 0;
 static int clockextra = 0;
@@ -699,7 +700,13 @@ static void __t1_configure_t1(struct t1 *wc, int lineconfig, int txlevel)
 	fmr2 = 0x22; /* FMR2: no payload loopback, auto send yellow alarm */
 	if (loopback)
 		fmr2 |= 0x4;
-	fmr4 = 0x0c; /* FMR4: Lose sync on 2 out of 5 framing bits, auto resync */
+
+	if (j1mode)
+		fmr4 = 0x1c;
+	else
+		fmr4 = 0x0c; /* FMR4: Lose sync on 2 out of 5 framing bits, auto resync */
+
+
 	lim2 = 0x21; /* LIM2: 50% peak is a "1", Advanced Loss recovery */
 	lim2 |= (mytxlevel << 6);	/* LIM2: Add line buildout */
 	__t1_framer_out(wc, 0x1d, fmr1);
@@ -721,6 +728,7 @@ static void __t1_configure_t1(struct t1 *wc, int lineconfig, int txlevel)
 		fmr2 |= 0xc0;
 	}
 	__t1_framer_out(wc, 0x1c, fmr0);
+
 	__t1_framer_out(wc, 0x20, fmr4);
 	__t1_framer_out(wc, 0x21, 0x40);	/* FMR5: Enable RBS mode */
 
@@ -733,6 +741,9 @@ static void __t1_configure_t1(struct t1 *wc, int lineconfig, int txlevel)
 	__t1_framer_out(wc, 0x3a, lim2);	/* LIM2: 50% peak amplitude is a "1" */
 	__t1_framer_out(wc, 0x38, 0x0a);	/* PCD: LOS after 176 consecutive "zeros" */
 	__t1_framer_out(wc, 0x39, 0x15);	/* PCR: 22 "ones" clear LOS */
+
+	if (j1mode)
+		__t1_framer_out(wc, 0x24, 0x80); /* J1 overide */
 		
 	/* Generate pulse mask for T1 */
 	switch(mytxlevel) {
@@ -885,7 +896,7 @@ static int t1xxp_shutdown(struct zt_span *span)
 	unsigned long flags;
 
 	spin_lock_irqsave(&wc->lock, flags);
-	t1_framer_out(wc, 0x46, 0x41);	/* GCR: Interrupt on Activation/Deactivation of AIX, LOS */
+	__t1_framer_out(wc, 0x46, 0x41);	/* GCR: Interrupt on Activation/Deactivation of AIX, LOS */
 	__t1xxp_stop_dma(wc);
 	__t1xxp_disable_interrupts(wc);
 	span->flags &= ~ZT_FLAG_RUNNING;
@@ -965,7 +976,7 @@ static int t1xxp_software_init(struct t1 *wc)
 		sprintf(wc->chans[x].name, "WCT1/%d/%d", wc->num, x + 1);
 		wc->chans[x].sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_EM_E1 | 
 				      ZT_SIG_FXSLS | ZT_SIG_FXSGS | 
-				      ZT_SIG_FXSKS | ZT_SIG_FXOLS | 
+				      ZT_SIG_FXSKS | ZT_SIG_FXOLS | ZT_SIG_DACS_RBS |
 				      ZT_SIG_FXOGS | ZT_SIG_FXOKS | ZT_SIG_CAS | ZT_SIG_SF;
 		wc->chans[x].pvt = wc;
 		wc->chans[x].chanpos = x + 1;
@@ -1532,6 +1543,7 @@ static struct pci_device_id t1xxp_pci_tbl[] = {
 	{ 0xe159, 0x0001, 0x79fe, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
 	{ 0xe159, 0x0001, 0x795e, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
 	{ 0xe159, 0x0001, 0x79de, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
+	{ 0xe159, 0x0001, 0x797e, PCI_ANY_ID, 0, 0, (unsigned long) "Digium Wildcard TE110P T1/E1 Board" },
 	{ 0 }
 };
 
@@ -1553,7 +1565,7 @@ static struct pci_driver t1xxp_driver = {
 static int __init t1xxp_init(void)
 {
 	int res;
-	res = pci_module_init(&t1xxp_driver);
+	res = zap_pci_module(&t1xxp_driver);
 	if (res)
 		return -ENODEV;
 	return 0;
@@ -1570,12 +1582,14 @@ module_param(loopback, int, 0600);
 module_param(t1e1override, int, 0600);
 module_param(clockextra, int, 0600);
 module_param(debug, int, 0600);
+module_param(j1mode, int, 0600);
 #else
 MODULE_PARM(alarmdebounce, "i");
 MODULE_PARM(loopback, "i");
 MODULE_PARM(t1e1override, "i");
 MODULE_PARM(clockextra, "i");
 MODULE_PARM(debug, "i");
+MODULE_PARM(j1mode, "i");
 #endif
 MODULE_DESCRIPTION("Wildcard TE110P Zaptel Driver");
 MODULE_AUTHOR("Mark Spencer <markster@digium.com>");
