@@ -2,7 +2,7 @@
 #define	CARD_FXS_H
 /*
  * Written by Oron Peled <oron@actcom.co.il>
- * Copyright (C) 2004-2005, Xorcom
+ * Copyright (C) 2004-2006, Xorcom
  *
  * All rights reserved.
  *
@@ -25,6 +25,24 @@
 #include "xpd.h"
 #include "slic.h"
 
+enum fxs_opcodes {
+	XPROTO_NAME(FXS, SIG_CHANGED)		= 0x06,
+/**/
+	XPROTO_NAME(FXS, SLIC_WRITE)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, CHAN_ENABLE)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, CHAN_POWER)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, CHAN_CID)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, RING)			= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, SETHOOK)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, LED)			= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, RELAY_OUT)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, SLIC_INIT)		= 0x0F,	/* Write to SLIC */
+	XPROTO_NAME(FXS, SLIC_QUERY)		= 0x0F,	/* Write to SLIC */
+/**/
+	XPROTO_NAME(FXS, SLIC_REPLY)		= 0x10,
+};
+
+
 DEF_RPACKET_DATA(FXS, SIG_CHANGED,
 	byte		type;		/* unused -- we have it from DEV_DESC */
 	xpp_line_t	sig_status;	/* channels: lsb=1, msb=8 */
@@ -34,10 +52,6 @@ DEF_RPACKET_DATA(FXS, SLIC_REPLY,	/* Get status of a single SLIC (for debugging)
 	xpp_line_t	lines;
 	slic_reply_t	info;
 	);
-DEF_RPACKET_DATA(FXS, SLIC_INIT,
-	xpp_line_t      lines;
-	slic_data_t     slic_data;
-	);
 DEF_RPACKET_DATA(FXS, SLIC_WRITE,
 	slic_cmd_t	slic_cmd;
 	);
