@@ -534,5 +534,20 @@ struct tone_zone builtin_zones[]  =
 		  { ZT_TONE_STUTTER, "450+425" },
 	  },
 	},
-	{ -1 }		
+        { 34, "ve", "Venezuela", { 1000, 4000 },
+          {
+            /* Reference: http://www.itu.int/ITU-T/inr/forms/files/tones-0203.pdf */
+            { ZT_TONE_DIALTONE, "425" },
+            { ZT_TONE_BUSY, "425/500,0/500" },
+            { ZT_TONE_RINGTONE, "425/1000,0/4000" },
+            { ZT_TONE_CONGESTION, "425/250,0/250" },
+            { ZT_TONE_CALLWAIT, "400+450/300,0/6000" },
+            { ZT_TONE_DIALRECALL, "425" },
+            { ZT_TONE_RECORDTONE, "1400/500,0/15000" },
+            { ZT_TONE_INFO, "!950/330,!1440/330,!1800/330,0/1000" },
+            /* STUTTER - not specified */
+            { ZT_TONE_STUTTER, "!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,!425/100,!0/100,425" },
+          },
+        },
+	{ -1 }
 };
