@@ -32,6 +32,7 @@
 #include <linux/kmod.h>
 #include <linux/sched.h>
 #include <linux/interrupt.h>
+#include <linux/vmalloc.h>
 #ifdef CONFIG_DEVFS_FS
 #include <linux/devfs_fs_kernel.h>
 #endif
@@ -125,9 +126,11 @@ static struct zt_dynamic_driver *drivers =  NULL;
 static int debug = 0;
 
 static int hasmaster = 0;
-
+#ifdef DEFINE_SPINLOCK
+static DEFINE_SPINLOCK(dlock); 
+#else
 static spinlock_t dlock = SPIN_LOCK_UNLOCKED;
-
+#endif
 
 static void checkmaster(void)
 {
@@ -407,7 +410,7 @@ static void dynamic_destroy(struct zt_dynamic *z)
 
 	/* Free channels */
 	if (z->chans);
-		kfree(z->chans);
+		vfree(z->chans);
 	/* Free z */
 	kfree(z);
 
@@ -556,7 +559,7 @@ static int create_dynamic(ZT_DYNAMIC_SPAN *zds)
 	memset(z, 0, sizeof(struct zt_dynamic));
 
 	/* Allocate other memories */
-	z->chans = kmalloc(sizeof(struct zt_chan) * zds->numchans, GFP_KERNEL);
+	z->chans = vmalloc(sizeof(struct zt_chan) * zds->numchans);
 	if (!z->chans) {
 		dynamic_destroy(z);
 		return -ENOMEM;
@@ -597,7 +600,7 @@ static int create_dynamic(ZT_DYNAMIC_SPAN *zds)
 		sprintf(z->chans[x].name, "ZTD/%s/%s/%d", zds->driver, zds->addr, x+1);
 		z->chans[x].sigcap = ZT_SIG_EM | ZT_SIG_CLEAR | ZT_SIG_FXSLS |
 				     ZT_SIG_FXSKS | ZT_SIG_FXSGS | ZT_SIG_FXOLS |
-				     ZT_SIG_FXOKS | ZT_SIG_FXOGS | ZT_SIG_SF;
+				     ZT_SIG_FXOKS | ZT_SIG_FXOGS | ZT_SIG_SF | ZT_SIG_DACS_RBS;
 		z->chans[x].chanpos = x + 1;
 		z->chans[x].pvt = z;
 	}
